@@ -5,7 +5,7 @@ oer+MySQL - IRC bot
 See ftp://nic.funet.fi/pub/unix/irc/docs/FAQ.gz section 11 for the
 definition of the word bot.
 
-Copyright (C) 2000-2002 EQU <equ@equnet.org>
+Copyright (C) 2000-2003 EQU <equ@equnet.org>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -183,112 +183,120 @@ void reconnect()
 
 int serverconnection()
 {
-	struct hostent *hptr;
-	struct sockaddr_in servaddr;
+	char service[TINYSTRINGLEN + 1];
+        struct addrinfo *res;
+        struct addrinfo *aip;
+        struct addrinfo hints;
+        int ai_error;
 #ifdef DEBUG
         oer_debug(OER_DEBUG_NOISE, "serverconnection()\n");
 #endif
-	/* check if we are already connected */
-	if(mystate->current_server != NULL) {
-		if(mystate->current_server->connected) {
-			return OER_SERVERCONNECTION_ALREADY_CONNECTED;
-		}
-	}
-	if((mystate->sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
-		oer_debug(OER_DEBUG_ERROR, "serverconnection->socket() failed: %s\n", strerror(errno));
-		return OER_SERVERCONNECTION_ERR_SOCKET;
-	}
-	memset(&servaddr, 0, sizeof(struct sockaddr_in));
-	servaddr.sin_family = AF_INET;
-	servaddr.sin_addr.s_addr = INADDR_ANY;
-	/* get system hostname */
+        /* check if we are already connected */
+        if(mystate->current_server != NULL) {
+                if(mystate->current_server->connected) {
+                        return OER_SERVERCONNECTION_ALREADY_CONNECTED;
+                }
+        }
+        /* get the system hostname */
         if(gethostname(mystate->host, HOSTLEN) < 0) {
                 oer_debug(OER_DEBUG_ERROR, "serverconnection->gethostname() failed: %s\n", strerror(errno));
-                close(mystate->sockfd);
                 return OER_SERVERCONNECTION_ERR_GETHOSTNAME;
         }
         if(strlen(mystate->vhost)) {
                 /* override with virtual host */
                 strncpy(mystate->host, mystate->vhost, HOSTLEN);
-        }
-	if((hptr = (struct hostent *) gethostbyname(mystate->host)) == NULL) {
-#ifndef _NO_H_ERRNO
-		oer_debug(OER_DEBUG_ERROR, "serverconnection->gethostbyname() for %s failed (h_errono = %d)\n", mystate->host, h_errno);
-		if(h_errno == TRY_AGAIN) {
-			oer_debug(OER_DEBUG_ERROR, "serverconnection->gethostbyname() non-fatal error\n");
-			close(mystate->sockfd);
-			return OER_SERVERCONNECTION_ERR_GETHOSTBYNAME_RETRY;
-		} else {
-			close(mystate->sockfd);
-			return OER_SERVERCONNECTION_ERR_GETHOSTBYNAME;
-		}
-#else
-		/* i'm feeling lucky, and lazy too :) */
-		oer_debug(OER_DEBUG_ERROR, "serverconnection->gethostbyname() unknown error\n");
-		close(mystate->sockfd);
-		return OER_SERVERCONNECTION_ERR_GETHOSTBYNAME_RETRY;
+#ifdef DEBUG
+		oer_debug(OER_DEBUG_INFO, "serverconnection->using hostname %s\n", mystate->host);
 #endif
-	}
-	memcpy((char *) &servaddr.sin_addr, hptr->h_addr, hptr->h_length);
-	if(strlen(mystate->vhost)) {
-		/* bind is only necessary for vhost */
-		if(bind(mystate->sockfd, (struct sockaddr *) &servaddr, sizeof(struct sockaddr)) < 0) {
-			oer_debug(OER_DEBUG_ERROR, "serverconnection->bind() failed: %s (check your vhost setting)\n", strerror(errno));
-			close(mystate->sockfd);
-			return OER_SERVERCONNECTION_ERR_BIND;
+                /* bind() is necessary only when we have a vhost */
+		memset(&hints, 0, sizeof(hints));
+		hints.ai_socktype = SOCK_STREAM;
+		if((ai_error = getaddrinfo(mystate->host, NULL, &hints, &res)) != 0) {
+			oer_debug(OER_DEBUG_ERROR, "serverconnection->getaddrinfo() for host %s failed: %s\n", mystate->host, gai_strerror(ai_error));
+			if(ai_error == EAI_AGAIN) {
+				oer_debug(OER_DEBUG_ERROR, "serverconnection->getaddrinfo() non-fatal error, retrying\n");
+				return OER_SERVERCONNECTION_ERR_GETHOSTBYNAME_RETRY;
+			} else {
+				return OER_SERVERCONNECTION_ERR_GETHOSTBYNAME;
+			}
+		}
+		for(aip = res; aip != NULL; aip = aip->ai_next) {
+			if((mystate->sockfd = socket(aip->ai_family, aip->ai_socktype, aip->ai_protocol)) < 0) {
+				oer_debug(OER_DEBUG_ERROR, "serverconnection->socket() failed: %s\n", strerror(errno));
+				freeaddrinfo(res);
+				return OER_SERVERCONNECTION_ERR_SOCKET;
+			}
+			if(bind(mystate->sockfd, aip->ai_addr, aip->ai_addrlen) < 0) {
+				oer_debug(OER_DEBUG_ERROR, "serverconnection->bind() for %s failed: %s (check your vhost setting)\n", mystate->host, strerror(errno));
+				close(mystate->sockfd);
+				freeaddrinfo(res);
+				return OER_SERVERCONNECTION_ERR_BIND;
+			}
+			/* we are satisfied */
+			break;
 		}
 	}
-	if((mystate->current_server = getserver()) == NULL) {
-		/* we didn't get a server this time but we will eventually
-		   once the used & stoned flags are cleared. Unless
-		   ofcourse the servers are permanently unreachable. */
-		close(mystate->sockfd);
-		return OER_SERVERCONNECTION_ERR_GETSERVER_RETRY;
-	}
+	freeaddrinfo(res);
+	/* get IRC server to connect to */
+        if((mystate->current_server = getserver()) == NULL) {
+                /* we didn't get a server this time but we will eventually
+                   once the used & stoned flags are cleared. Unless
+                   ofcourse the servers are permanently unreachable. */
+                return OER_SERVERCONNECTION_ERR_GETSERVER_RETRY;
+        }
         memset(mystate->current_server->serverhost_r, 0, HOSTLEN + 1);
-	mystate->current_server->registerconnection_start = mystate->now;
-	mystate->current_server->used = 1;
+        mystate->current_server->registerconnection_start = mystate->now;
+        mystate->current_server->used = 1;
 #ifdef DEBUG
-	oer_debug(OER_DEBUG_INFO, "serverconnection->using server %s port %d\n", mystate->current_server->serverhost, mystate->current_server->serverport);
-#endif
-	if((hptr = (struct hostent *) gethostbyname(mystate->current_server->serverhost)) == NULL) {
-#ifndef _NO_H_ERRNO
-		oer_debug(OER_DEBUG_ERROR, "serverconnection->gethostbyname() for %s failed (h_errno = %d)\n", mystate->current_server->serverhost, h_errno);
-		if(h_errno == NO_ADDRESS || h_errno == HOST_NOT_FOUND || h_errno == TRY_AGAIN) {
-#ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "serverconnection->switching to next server if available\n");
-#endif
-			close(mystate->sockfd);
-			return OER_SERVERCONNECTION_ERR_GETHOSTBYNAME_RETRY;
-		}
-#else
-		/* i'm feeling lucky, and lazy too :) */
-		oer_debug(OER_DEBUG_ERROR, "serverconnection->gethostbyname() unknown error\n");
-		close(mystate->sockfd);
-		return OER_SERVERCONNECTION_ERR_GETHOSTBYNAME_RETRY;
+        oer_debug(OER_DEBUG_INFO, "serverconnection->using IRC server %s port %d\n", mystate->current_server->serverhost, mystate->current_server->serverport);
 #endif
+        snprintf(service, TINYSTRINGLEN, "%d", mystate->current_server->serverport);
+        memset(&hints, 0, sizeof(hints));
+        hints.ai_socktype = SOCK_STREAM;
+	if((ai_error = getaddrinfo(mystate->current_server->serverhost, service, &hints, &res)) != 0) {
+                oer_debug(OER_DEBUG_ERROR, "serverconnection->getaddrinfo() for host %s service %s failed: %s\n", mystate->current_server->serverhost, service, gai_strerror(ai_error));
+                if(ai_error == EAI_AGAIN) {
+                        oer_debug(OER_DEBUG_ERROR, "serverconnection->getaddrinfo() non-fatal error, retrying\n");
+                        return OER_SERVERCONNECTION_ERR_GETHOSTBYNAME_RETRY;
+                } else {
+                        return OER_SERVERCONNECTION_ERR_GETHOSTBYNAME;
+                }
         }
-	memcpy(&servaddr.sin_addr, hptr->h_addr_list[0], sizeof(struct in_addr));
-	servaddr.sin_port = htons(mystate->current_server->serverport);
-	if(connect(mystate->sockfd, (struct sockaddr *) &servaddr, sizeof(struct sockaddr_in)) < 0) {
-		oer_debug(OER_DEBUG_ERROR, "serverconnection->connect() failed: %s (%d)\n", strerror(errno), errno);
-		if(errno == ECONNREFUSED || errno == ETIMEDOUT || errno == ENETUNREACH || errno == EHOSTUNREACH) {
+	for(aip = res; aip != NULL; aip = aip->ai_next) {
+		if(!strlen(mystate->vhost)) {
+                        /* we need to create the socket because it wasn't
+                           created before for bind() */
+			if((mystate->sockfd = socket(aip->ai_family, aip->ai_socktype, aip->ai_protocol)) < 0) {
+                                oer_debug(OER_DEBUG_ERROR, "serverconnection->socket() failed: %s\n", strerror(errno));
+                                freeaddrinfo(res);
+                                return OER_SERVERCONNECTION_ERR_SOCKET;
+                        }
+                }
+                if(connect(mystate->sockfd, aip->ai_addr, aip->ai_addrlen) < 0) {
+                        oer_debug(OER_DEBUG_ERROR, "serverconnection->connect() failed: %s (%d)\n", strerror(errno), errno);
+                        if(errno == ECONNREFUSED || errno == ETIMEDOUT || errno == ENETUNREACH || errno == EHOSTUNREACH) {
 #ifdef DEBUG
-			oer_debug(OER_DEBUG_INFO, "serverconnection->connect() non-fatal error\n");
-			oer_debug(OER_DEBUG_INFO, "serverconnection->switching to next server if available\n");
+                                oer_debug(OER_DEBUG_INFO, "serverconnection->connect() non-fatal error\n");
+                                oer_debug(OER_DEBUG_INFO, "serverconnection->switching to next server if available\n");
 #endif
-			close(mystate->sockfd);
-			return OER_SERVERCONNECTION_ERR_CONNECT_RETRY;
-		}
-		close(mystate->sockfd);
-		return OER_SERVERCONNECTION_ERR_CONNECT;
-	}
-	proxysetup();
-	mystate->current_server->connected = 1;
-	mystate->current_server->linkup = mystate->now;
-	mystate->current_server->rx = 0;
-	mystate->current_server->tx = 0;
-	return OER_SERVERCONNECTION_EVERYTHING_OK;
+                                close(mystate->sockfd);
+                                freeaddrinfo(res);
+                                return OER_SERVERCONNECTION_ERR_CONNECT_RETRY;
+                        }
+                        close(mystate->sockfd);
+                        freeaddrinfo(res);
+                        return OER_SERVERCONNECTION_ERR_CONNECT;
+                }
+                /* we are satisfied */
+                break;
+        }
+        freeaddrinfo(res);
+        proxysetup();
+        mystate->current_server->connected = 1;
+        mystate->current_server->rx = 0;
+        mystate->current_server->tx = 0;
+        mystate->current_server->linkup = mystate->now;
+        return OER_SERVERCONNECTION_EVERYTHING_OK;
 }
 
 int registerconnection()
